/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl> and
  Sheldon Lee-Wen <leewsb@hotmail.com>
  http://lineak.sourceforge.net
                                                                        
  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifndef _LINEAKD_H
#define _LINEAKD_H

/* definition and configuration file parser */
#include "dcfp.h"

/* LinEAK common include */
#include "lineakcommon.h"

/* structures where conf and def data goes */
static const dcfp_data_struct *confdata = NULL;
static const dcfp_data_struct *defdata = NULL;
static const dcfp_data_struct *usrdefdata = NULL;

/* function prototypes */
void showusage(int argc, char *argv[]);
int parseargs(int argc, char *argv[]);
void showkeyboards (void);
int initEAK (void);
void execEAcommand(int keycode);
EAkey *lookupEAkey(int keycode);
void parseconffile(void);
void create_new_conf(char *homedir, char *type);
void parsedeffile(void);
int file_exists(char *filename);
void delete_file(char *filename);
void check_pid_on_startup(void);
static void cleanexit (int status);
static void cleanup (void);
void cleanKeylist (void);
static void signalexit(int sig_num);
static void signalHUP(int sig_num);
void initializeX(void);
void cleanupX(void);
Display *GetDisplay(char* dpyName, int* opcodeRtrn, int* evBaseRtrn);
void commitXKBChanges(int tcode);
void initCDROM (void);
void DoEject(void);
void initSound(void);
int AdjustVolume(int value);

/* miscellaneous global variables and init */
static int xkbremapped = 0; /* If remapped by something else, set to 1, if reclamed by another
			       call to initializeX() the set to false */
static int runasdaemon = 0;
static int verbosemode = 0;
static int cdromstatus = 0;
static int ismuted = 0;
static int old_mastervol = 0;
static int nrretries = 0;
#define MAXRETRIES 25 /* prevents possible race condition, don't set this too low!! 25 will do */
static char *programname = NULL;
static char *deffilename = NULL;
static char *usrdeffilename = NULL;
static char *conffilename = NULL;
static EAKeyboard myEAK = { NULL, NULL, NULL, NULL }; /* the keyboard we'll use */

/* global xosd related stuff */
#ifdef HAVE_XOSD
xosd *osd;

char *font = "fixed";
char *color = "green";
int delay = 4;
int forcewait=0;
xosd_pos pos = XOSD_bottom;
int offset = 20;
int shadow = 2;
int scroll_age = 0;
struct timeval old_age,new_age;
int screen_line = 0;
int lines=2;
static int max_audio = 25700;
/* xods realted functions */
xosd* init_lxosd(char *font, char*color, int delay, xosd_pos pos, int offset, int shadow, int lines);
void cleanup_lxosd(xosd *osd);
void lxosd_agenscroll(void);
void lxosd_age(void);
void lxosd_volume(int volume);
void lxosd_show(char *command);
#endif


/* global Xkb related stuff */
Display *ourdisplay;
XkbDescPtr xkb;
char *displayname = NULL;
int xkbOpcode;
int xkbEventCode;
static int caughtXerror = 0;

/* some globals from the .conf */
char *kbtype=NULL, *mixdev=NULL, *cdromdev=NULL;

/* special actions in .conf file */
#define EAK_EJECT "EAK_EJECT"
#define EAK_VOLUP "EAK_VOLUP"
#define EAK_VOLDOWN "EAK_VOLDOWN"
#define EAK_MUTE "EAK_MUTE"
#define EAK_SLEEP "EAK_SLEEP"

/* A System wide config file */
#define SYS_CONFFILE "/etc/lineak/lineakd.conf"

/* for volume settings using the mixer device */
#define MIXERDEV "/dev/mixer"
#define VOLUP_VALUE 5
#define VOLDOWN_VALUE -5
#define VOLMUTE_VALUE 0
#define MAX_VOLUME 100

/* we might need this .. oss emulation under BSD */
#if defined (__NetBSD__) || defined (__OpenBSD__)
#define SOUND_IOCTL(a,b,c)      _oss_ioctl(a,b,c)
#else
#define SOUND_IOCTL(a,b,c)      ioctl(a,b,c)
#endif

#define CONF_HEADER \
"# LinEAK - Linux support for Easy Access and Internet Keyboards\n" \
"#  Copyright (c) 2001,2002 Mark Smulders <Mark@PIRnet.nl>\n" \
"#  http://lineak.sourceforge.net\n" \
"#\n" \
"# lineakd configuration file\n" \
"#\n" \
"# example key configuration:\n" \
"# \tplay\t= \"xmms --play-pause\"\n" \
"# \teject\t= EAK_EJECT\n" \
"#\n" \
"# available special actions:\n" \
"# \tEAK_EJECT\n" \
"# \tEAK_VOLUP\n" \
"# \tEAK_VOLDOWN\n" \
"# \tEAK_MUTE\n" \
"# \tEAK_SLEEP\n" \
"#\n" \
"\n"

#endif
